'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
// istanbul ignore next

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

// istanbul ignore next

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

// istanbul ignore next

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

// istanbul ignore next

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _postcss = require('postcss');

var _reduceFunctionCall = require('reduce-function-call');

var _reduceFunctionCall2 = _interopRequireDefault(_reduceFunctionCall);

var _parseFilepath = require('parse-filepath');

var _parseFilepath2 = _interopRequireDefault(_parseFilepath);

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _jsYaml = require('js-yaml');

var _jsYaml2 = _interopRequireDefault(_jsYaml);

var reMap = /((?:map)\()(.*)(\))/;
var defaultMap = 'config';

var Parser = (function () {
  function Parser() {
    var _ref = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var _ref$basePath = _ref.basePath;
    var basePath = _ref$basePath === undefined ? process.cwd() : _ref$basePath;
    var _ref$maps = _ref.maps;
    var maps = _ref$maps === undefined ? [] : _ref$maps;

    _classCallCheck(this, Parser);

    this.basePath = basePath;
    this.maps = {};

    maps.forEach(this.getMap.bind(this));
  }

  /**
   * Parse and replace maps in declarations values.
   * @param {Object} decl
   */

  _createClass(Parser, [{
    key: 'processDecl',
    value: function processDecl(decl) {
      // istanbul ignore next

      var _this = this;

      if (!reMap.test(decl.value)) return;

      decl.value = (0, _reduceFunctionCall2['default'])(decl.value, 'map', function (body) {
        return _this.getValue(_postcss.list.comma(body));
      });
    }

    /**
     * Parse and proccess at-rules.
     * @param {Object} rule
     */
  }, {
    key: 'processAtRule',
    value: function processAtRule(rule) {
      if (rule.name === 'map') return this.replaceAtRuleBlock(rule);
      if (reMap.test(rule.params)) return this.replaceAtRuleParam(rule);
    }

    /**
     * Get and print at-rules map declarations.
     * @param {Object} rule
     */
  }, {
    key: 'replaceAtRuleBlock',
    value: function replaceAtRuleBlock(rule) {
      var map = this.getValue(_postcss.list.space(rule.params));

      Object.keys(map).forEach(function (prop) {
        rule.parent.insertBefore(rule, { prop: prop, value: map[prop] });
      });

      rule.remove();
    }

    /**
     * Parse and replace maps in at-rules parameters.
     * @param {Object} rule
     */
  }, {
    key: 'replaceAtRuleParam',
    value: function replaceAtRuleParam(rule) {
      // istanbul ignore next

      var _this2 = this;

      rule.params = (0, _reduceFunctionCall2['default'])(rule.params, 'map', function (body) {
        return _this2.getValue(_postcss.list.comma(body));
      });
    }

    /**
     * Get value from a deep nested object properties.
     * @param {Array} args
     * @return {*}
     */
  }, {
    key: 'getValue',
    value: function getValue(args) {
      var _args = _toArray(args);

      var name = _args[0];

      var props = _args.slice(1);

      var shortcutMap = this.useShortcutMap(name);

      if (shortcutMap) {
        name = shortcutMap;
        props = args;
      }

      var value = this.getMap(name);

      props.forEach(function (prop) {
        value = value[prop];
      });

      return value;
    }

    /**
     * Get map name usable with the short syntax.
     * @param {String} name
     * @return {Boolean|String}
     */
  }, {
    key: 'useShortcutMap',
    value: function useShortcutMap(name) {
      if (name in this.maps) return false;
      if (defaultMap in this.maps) return defaultMap;
      var names = Object.keys(this.maps);
      if (names.length === 1) return names[0];
    }

    /**
     * Load YAML or JSON from given file.
     *
     * @param {String} file
     * @return {Object}
     */
  }, {
    key: 'parseFile',
    value: function parseFile(file) {
      try {
        var filePath = _path2['default'].resolve(this.basePath, file);
        return _jsYaml2['default'].safeLoad(_fs2['default'].readFileSync(filePath, 'utf-8'));
      } catch (e) {
        console.error(e);
      }
    }

    /**
     * Get a cached map or parse it.
     *
     * @param {String} map
     * @return {Object}
     */
  }, {
    key: 'getMap',
    value: function getMap(map) {
      if (typeof map === 'object') {
        return (0, _objectAssign2['default'])(this.maps, map);
      }

      var _parsePath = (0, _parseFilepath2['default'])(map);

      var name = _parsePath.name;
      var basename = _parsePath.basename;

      return this.maps[name] || (this.maps[name] = this.parseFile(basename));
    }
  }]);

  return Parser;
})();

exports['default'] = Parser;
module.exports = exports['default'];